# $Id: drilldown.cfg 2905 2010-01-15 11:52:13Z anton $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/trunk/sources/meresco/bzv/etc/drilldown.cfg $
from bzv.simplejson.decoder import JSONDecoder
from bzv.simplejson.encoder import JSONEncoder
from cgi import parse_qs
from cqlparser import parseString as parseCQL
from meresco.core import Observable
from urlparse import urlsplit
from httplib import HTTPConnection, OK
import urllib
import pprint
#pp = pprint.PrettyPrinter(indent=2)

#===============================================================================
# ServerSide
#===============================================================================
class Request(Observable):
    def __init__(self, infra):
        Observable.__init__(self)
        self.infra = infra
        self.logger = infra.logging.getLogger('meresco.json')
        self.logger.debug("starting logger")
        self.pp = pprint.PrettyPrinter(indent=2)

    def okJSON(self):
        CRLF = "\r\n"
        ContentTypeJSON = "Content-Type: application/json"
        Ok = "HTTP/1.0 200 Ok" + CRLF
        okJSON = "HTTP/1.0 200 OK" + CRLF + \
                 ContentTypeJSON + CRLF + \
                 CRLF
        return okJSON

    def extractRemoteDrilldownConfig(self, jsonConfig):
        try:
            drilldownConfigRequest = JSONDecoder().decode(jsonConfig)
            drilldownFieldFeatures = drilldownConfigRequest['drilldownFieldFeatures']
            multiLevelDrilldownDict = drilldownConfigRequest['multiLevelDrilldownDict']

        except:
            self.logger.error ('Error in JSONDecoder, decoding: ' + pp.pformat(jsonConfig))

        # Note: the returned data contain unicode. Conversion to str is needed for diacritics
        self.logger.debug("drilldownFieldFeatures" + self.pp.pformat(drilldownFieldFeatures))
        self.logger.debug("multiLevelDrilldownDict" + self.pp.pformat(multiLevelDrilldownDict))
        return (drilldownFieldFeatures, multiLevelDrilldownDict)


    def handleRequest(self, RequestURI='', **kwargs):
        pp = self.pp
        self.logger.debug("kwargs: " + pp.pformat(kwargs))

        Scheme, Netloc, Path, Query, Fragment = urlsplit(RequestURI)
        arguments = parse_qs(Query)

        jsonDrilldownConfigRequest = arguments.get('jsonDrilldownConfigRequest', [''])[0]
        (remoteDrilldownFieldFeatures, remoteMultiLevelDrilldownDict) =  self.extractRemoteDrilldownConfig(jsonDrilldownConfigRequest)

        query =  arguments.get('query', [''])[0]
        drilldownArguments = arguments.get('drilldown', [])

        extra = arguments.get('extra', [])
        parsetree, cqlQuery = self.any.parseQuery(query, drilldownArguments, extra)
        start = int(arguments.get('start', ['0'])[0])
        stop = int(arguments.get('stop', ['10'])[0])
        sortBy =  arguments.get('sortBy', [''])[0]
        sortDescending =  arguments.get('sortDescending', ['1'])[0]
        sortDescending = sortDescending == '1'
        total, recordIds = self.any.executeCQL(cqlAbstractSyntaxTree=parsetree, start=start, stop=stop, sortBy=sortBy, sortDescending=sortDescending)
        records = {}
        for recordId in recordIds:
            record = self.any.createRecord(recordId)
            records[recordId] = record
        docset = self.any.docsetFromQuery(cqlAbstractSyntaxTree=parsetree)
        drilldownFields = self.any.getDrilldownFields('drilldownFields')

        drilldownFieldList = []
        for el in remoteDrilldownFieldFeatures:
            (req,min,sort) = remoteDrilldownFieldFeatures[el]
            drilldownFieldList.append( (str(el),req, sort))

        # n.b. drilldownResults is a generator
        self.logger.debug("drilldownFieldList: " + self.pp.pformat(drilldownFieldList))
        self.logger.debug("drilldownFields: " + self.pp.pformat(drilldownFields))
        drilldownResults = self.any.drilldown(docset, drilldownFieldList)

        drilldownresults = {}
        for fieldname, termCounts in drilldownResults:
            facet = []
            for term, count in termCounts:
                facet.append([str(term),str(count)])
            drilldownresults[fieldname] = facet

#        self.logger.debug("drilldownresults: " + self.pp.pformat(drilldownresults) )

        multilevelResults = self.any.multiLevelDrilldown(docset, ['date', 'author'], remoteMultiLevelDrilldownDict, remoteDrilldownFieldFeatures)
        multilevelresults = {}
        for (fieldName, levelField), termCounts in multilevelResults:
            facet = []
            for term, count in termCounts:
                facet.append([str(term),str(count)])
            multilevelresults[fieldName] = {'level':str(levelField), 'termCounts':facet}

        response = {'total':total, 'recordIds':recordIds,
                    'records':records, 'drilldownResults':drilldownresults,
                    'multilevelResults':multilevelresults}

        yield self.okJSON()
        yield JSONEncoder().encode(response)

#===============================================================================
# ClientSide
#===============================================================================
class Response(Observable):
    def __init__(self, infra):
        Observable.__init__(self)
        self.infra = infra
        self.logger = infra.logging.getLogger('meresco.json')
        self.pp = pprint.PrettyPrinter(indent=2)

    def getResponse(self, query='', drilldownArguments=[], extra=[], dbserver='self', start=1, stop=11, sortBy='drilldown.norm.date.year', sortDescending='1', infra = None):
        logger = infra.logger
        json = '{}'
        pp =  self.pp

        drilldownConfigRequest = {}
        drilldownConfigRequest['drilldownFieldFeatures']  = self.any.getFieldFeatures()
        drilldownConfigRequest['multiLevelDrilldownDict'] = self.any.getMultiLevelDrilldownFields()

        try:
            jsonDrilldownConfigRequest = JSONEncoder().encode(drilldownConfigRequest)
        except:
            logger.error('Error in JSONEncoder, encoding: ' + pp.pformat(drilldownConfigRequest))

        if dbserver[0:4] != 'http':
            json = '{"diagnostic": "no http or https url"}'
        elif query:
            Query = [("query",query), ("start",str(start)), ("stop",str(stop)),
                     ("sortBy",sortBy), ("sortDescending",sortDescending),
                     ("jsonDrilldownConfigRequest", jsonDrilldownConfigRequest)
                     ]
            if drilldownArguments:
                 for drilldown in drilldownArguments:
                     Query.append(("drilldown", drilldown))
            if extra:
                 for e in extra:
                     Query.append(('extra', extra))
            request = '/JSON/searchresult?' + urllib.urlencode(Query)
            url = dbserver + request
            try:
                serverresponse = urllib.urlopen(url)
                json = serverresponse.read()
            except Exception:
                logger.error("Could not read from url: " + str(url))
                json = '{"total":0}'
        else:
            json = '{"total":0}'

        try:
            decoded = JSONDecoder().decode(json)

        except Exception:
            logger.error ('Error in JSONDecoder, decoding: ' + json )
            json = '{"total":0}'
            decoded = JSONDecoder().decode(json)

#        self.logger.debug(infra.prpt('json', json))
        return decoded

